/* ===========================================================================
 * IBA CZ Confidential
 *
 * © Copyright IBA CZ 2009 ALL RIGHTS RESERVED
 * The source code for this program is not published or otherwise
 * divested of its trade secrets.
 *
 * =========================================================================== */ 

package eu.ibacz.pv230.simpleshop.backend.dto;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang.Validate;

/**
 *
 * @author Rastislav Papp
 */
public final class BasketDTO {

    private final String userId;
    private final List<BasketItemDTO> items;

    public BasketDTO(String userId, List<BasketItemDTO> items) {
        Validate.notNull(items);
        Validate.notEmpty(userId);
        
        this.userId = userId;
        this.items = items;
    }

    public List<BasketItemDTO> getItems() {
        return Collections.unmodifiableList(items);
    }
    
    public int getItemCount() {
        int size = 0;
        for (BasketItemDTO item : items) {
            size += item.getAmount();
        }
        return size;
    }
    
    public BigDecimal getTotalPrice() {
        BigDecimal price = BigDecimal.ZERO;
        for (BasketItemDTO item : items) {
            price = price.add(item.getPrice());
        }
        return price;
    }

    public String getUserId() {
        return userId;
    }

}
